from __future__ import annotations

import logging
import time
from pathlib import Path
from typing import List, Optional

import typer

from to7z import __version__, to7z

logger = logging.getLogger("to7z")
logging.basicConfig(level=logging.DEBUG)


PROG_NAME = "to7z"
app = typer.Typer()


def human_size(byts: int) -> str:
    rep: str = "bytes"
    num: float = byts

    if num > 1024:
        num /= 1024.0
        rep = "KB"

    if num > 1024:
        num /= 1024.0
        rep = "MB"

    if num > 1024:
        num /= 1024.0
        rep = "GB"

    return f"{num:.2f} {rep}"


def version_callback(value: bool) -> None:
    if value:
        typer.echo(f"to7z version: {__version__}")
        raise typer.Exit()


@app.command()
def cli(
    path: Path = typer.Argument(
        Path(".").resolve(),
        show_default=False,
        help="Path to working directory. Default to current dir",
    ),
    ignore: List[Path] = typer.Option(
        [], "-i", "--ignore", help="Directories to ignore"
    ),
    recursive: bool = typer.Option(False, "-r", "--recursive"),
    fetch: bool = typer.Option(
        False,
        "-f",
        "--fetch",
        help="Return list of zips and rar found in path",
        is_flag=True,
    ),
    keep_original_files: bool = typer.Option(
        False,
        "-k",
        "--keep-original",
        is_flag=True,
        help="Keep original ZIP files. Default is to delete them",
    ),
    debug: bool = typer.Option(False, "--debug", "-d", is_flag=True),
    version: Optional[bool] = typer.Option(
        None, "--version", "-v", callback=version_callback, is_eager=True
    ),
) -> None:

    logger.setLevel(logging.INFO)
    if debug:
        logger.setLevel(logging.DEBUG)

    if not path.is_dir():
        logger.error("Path not found!")
        return

    ig: Path
    for ig in ignore:
        if not ig.is_dir():
            logger.error(f"Dir {ig} in IGNORE not found")
            return

    logger.info("Starting program")

    logger.info("Fetching files...")
    rar_and_zips = to7z.get_rar_and_zips(
        path.resolve(), ignore=set(ig.resolve() for ig in ignore), recursive=recursive
    )
    if len(rar_and_zips) == 0:
        logger.info("None found!")
        return
    elif len(rar_and_zips) == 1:
        logger.info(f"Found 1 file")
    else:
        logger.info(f"Found {len(rar_and_zips)} files")

    start_time = time.time()
    zips_total_size = to7z.get_total_size(rar_and_zips)

    logger.info(f"Total size: {human_size(zips_total_size)}")

    if fetch:
        file: Path
        for i, file in enumerate(rar_and_zips):
            typer.echo(f"{i+1:02d} - {human_size(file.stat().st_size)} - {file}")
        resp = input("Proceed? [y/N] ").lower()
        if not (resp in ("y", "s", "yes", "sim")):
            return

    if not debug:
        with typer.progressbar(rar_and_zips, label="Processing") as files_progress:
            for file in files_progress:
                to7z.change_to_7z(file)
    else:
        to7z.change_to_7z_iterate(rar_and_zips)

    f: Path
    total_size_7z = to7z.get_total_size(
        set([f.with_name(f.name.replace(f.suffix, ".7z")) for f in rar_and_zips])
    )

    logger.info(f"Total size after: {human_size(total_size_7z)}")

    if total_size_7z < zips_total_size:
        logger.info(
            f"Reduced {100.*(zips_total_size- total_size_7z)/zips_total_size:.2f}% of size"
        )

    if keep_original_files:
        logger.info("Keeping original files")
    else:
        logger.info("Deleting original files")
        if not debug:
            with typer.progressbar(rar_and_zips, label="Deleting") as files_progress:
                for file in files_progress:
                    to7z.delete_rar_and_zip(file)
        else:
            to7z.delete_rar_and_zips_iterate(rar_and_zips)

    elapsed_time = time.time() - start_time
    time_repr = f"{elapsed_time:.2f} seconds"
    if elapsed_time > 60:
        time_repr = (
            f"{elapsed_time//60.0:.0f} minute(s) {elapsed_time % 60.0:.0f} seconds"
        )
        elapsed_time /= 60.0
    logger.info(f"Finished in {time_repr}")


def main() -> None:
    app(prog_name=PROG_NAME)


if __name__ == "__main__":
    main()



from __future__ import annotations

import argparse
import logging
import os
import shutil
import time
import zipfile
from pathlib import Path
from typing import Set, TypedDict

# import rarfile
import py7zr

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.DEBUG)


class Args(TypedDict):
    source: Path
    recursive: bool


def get_from_args() -> Args:
    """Get command ling arguments from sys args

    Raises:
        RuntimeError: If args.source is not informed

    Returns:
        Args: dict
            Args["source"]: Path
            Args["recursive"]: bool
    """
    ret: Args = {"source": Path("."), "recursive": False}
    parser = argparse.ArgumentParser("to7z")
    parser.add_argument("source", type=Path)
    parser.add_argument("--debug", action="store_true")
    parser.add_argument("-r", "--recursive", action="store_true")
    args = parser.parse_args()
    if args.source is None:
        raise RuntimeError("Expected source arg")

    logger.setLevel(logging.INFO)
    if args.debug:
        logger.setLevel(logging.DEBUG)

    logger.debug(f"Arguments: {args}")

    src = Path(args.source).resolve()

    ret["source"] = src
    ret["recursive"] = args.recursive

    return ret


def is_rar_or_zip(file: Path) -> bool:
    """Return True if file is a .zip

    Args:
        file (Path): File to check

    Returns:
        bool: True if file is .zip
    """
    suffix = file.suffix.lower()
    return suffix in (".zip")


def not_in_ignore(file: Path, ignore: Set[Path]) -> bool:
    """Check if file is not in a ignored directory

    Args:
        file (Path): File to check
        ignore (Set[Path]): Set containing ignored directories

    Returns:
        bool: True if file is not contained int ignore directories
    """
    root_dir = str(file.parent.resolve())
    for i in ignore:
        if str(i.resolve()) in root_dir:
            return False
    return True


def get_rar_and_zips(
    source: Path, ignore: Set[Path] = set(), recursive: bool = False
) -> Set[Path]:
    """Get all .rar and .zips files from a given path

    Args:
        source (Path): Working path
        ignore (Set[Path], optional): Directories to be ignored. Defaults to set().
        recursive (bool, optional): If True, performs a recursive search. Defaults to False.

    Returns:
        Set[Path]: Set containg all .zip and .rar found
    """
    rar_and_zips: Set[Path] = set()

    for root, dirs, files in os.walk(source, topdown=True):
        for file in files:
            full_file = Path(root) / file
            if is_rar_or_zip(full_file) and not_in_ignore(full_file, ignore):
                logger.debug(f"Found file {full_file}")
                rar_and_zips.add(full_file)
        if not recursive:
            break

    return rar_and_zips


def extract_zip(file: Path, dest: Path) -> None:
    """Extract file .zip contents to dest folder

    Args:
        file (Path): .zip file to be extracted
        dest (Path): Destination of extracted content
    """
    logger.debug(f"Extracting zip {file} to {dest}")
    with zipfile.ZipFile(file, "r") as zip_ref:
        zip_ref.extractall(dest)


def extract_rar(file: Path, dest: Path) -> None:
    """Extract file .rar contents to dest folder

    Args:
        file (Path): .rar file to be extracted
        dest (Path): Destination of extracted content
    """
    logger.debug(f"Extracting rar {file} to {dest}")
    logger.debug(f"RAR: not implemented!")


def create_7z_from_dir(src: Path, file_7z: Path) -> None:
    """Create a .7z from a directory

    Args:
        src (Path): Directory to be compacted. All contents will be compacted.
        file_7z (Path): .7z file (with extension). Ex.: "output.7z"
    """
    cur_dir = Path.cwd()
    with py7zr.SevenZipFile(file_7z, "w") as archive:
        logger.debug(f"Changing dir to {src}")
        os.chdir(src)
        archive.writeall(".")
        os.chdir(cur_dir)
    logger.debug(f"Created {file_7z}")


def change_to_7z(file: Path) -> None:
    """Convert .rar or .zip file to .7z

    Args:
        file (Path): .zip or .7z file
    """
    temp_folder = Path(str(file) + "_temp_dir")
    logger.debug(f"Creating temp dir {temp_folder}")
    temp_folder.mkdir(exist_ok=True)

    suffix = file.suffix.lower()
    if suffix == ".zip":
        extract_zip(file, temp_folder)
    elif suffix == ".rar":
        extract_rar(file, temp_folder)

    basename_file_7z = file.name.replace(file.suffix, ".7z")
    file_7z = file.with_name(basename_file_7z)

    logger.debug(f"Creating 7z from {temp_folder} to {file_7z}")
    create_7z_from_dir(temp_folder, file_7z)

    logger.debug(f"Removing temp dir {temp_folder}")
    shutil.rmtree(temp_folder, ignore_errors=True)


def change_to_7z_iterate(files: Set[Path]) -> None:
    """Converts list of .rar and .zip files to .7z files

    Args:
        files (Set[Path]): Set containing all .rar and .zip files to be converted
    """
    for file in files:
        change_to_7z(file)


def delete_rar_and_zip(file: Path, check_if_7z_exists: bool = True) -> None:
    """Delete file. Optionally, checks if there's a .7z with the same name.

    Args:
        file (Path): .rar or .zip file to be deleted
        check_if_7z_exists (bool, optional): Checks if there's a correspondent .7z file before delting the .zip. Defaults to True.
    """
    if check_if_7z_exists:
        file7z = file.with_name(file.name.replace(file.suffix, ".7z"))
        if file7z.is_file():
            logger.debug(f"Safely removing file {file}")
            os.remove(file)
    else:
        logger.debug(f"Removing file {file}")
        os.remove(file)


def delete_rar_and_zips_iterate(
    files: Set[Path], check_if_7z_exists: bool = True
) -> None:
    """Calls delete_rar_and_zip in all files

    Args:
        files (Set[Path]): Files to be deleted
        check_if_7z_exists (bool, optional): Defaults to True.
    """
    for file in files:
        delete_rar_and_zip(file, check_if_7z_exists=check_if_7z_exists)


def get_total_size(files: Set[Path]) -> int:
    """Returns the total size (in bytes) of files.

    Args:
        files (Set[Path]): Files to be analyzed

    Returns:
        int: Size (in bytes) of all files
    """
    total_size: int = 0
    for file in files:
        total_size += file.stat().st_size
    return total_size


def main() -> None:
    logger.info("Starting program")
    start_time = time.time()

    args = get_from_args()
    source = args["source"]
    recursive = args["recursive"]

    rar_and_zips = get_rar_and_zips(source, recursive=recursive)
    change_to_7z_iterate(rar_and_zips)
    delete_rar_and_zips_iterate(rar_and_zips)

    elapsed_time = time.time() - start_time
    logger.info(f"Finished in {elapsed_time:.2f} seconds")



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['to7z']

package_data = \
{'': ['*']}

install_requires = \
['py7zr>=0.16.1,<0.17.0', 'typer[all]>=0.3.2,<0.4.0']

entry_points = \
{'console_scripts': ['to7z = to7z.cli:main']}

setup_kwargs = {
    'name': 'to7z',
    'version': '1.1.4',
    'description': 'Converts ZIP to 7z.',
    'long_description': '# `to7z`\n\nConverts ZIP to 7z.\n\n**Usage**:\n\n```console\n$ to7z [OPTIONS] [PATH]\n```\n\n**Arguments**:\n\n* `[PATH]`: Path to working directory. Default to current dir\n\n**Options**:\n\n* `-i, --ignore PATH`: Directories to ignore  [default: ]\n* `-r, --recursive`: [default: False]\n* `-f, --fetch`: Return list of zips and rar found in path  [default: False]\n* `-k, --keep-original`: Keep original ZIP files. Default is to delete them  [default: False]\n* `-d, --debug`: [default: False]\n* `-v, --version`\n* `--install-completion`: Install completion for the current shell.\n* `--show-completion`: Show completion for the current shell, to copy it or customize the installation.\n* `--help`: Show this message and exit.\n',
    'author': 'Marcus Bruno Fernandes Silva',
    'author_email': 'marcusbfs@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
